//--------------------------------------------------------------------------------------------------
//  Copyright Joe Kraska, 2006. This program is free software. You can redistribute it
//  and/or modify it under terms of the GNU General Public License as published by the
//  Free Software Foundation; either version 2 of the License, or (at your option) any
//  later version. See COPYING.TXT for details.
//--------------------------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
//--------------------------------------------------------------------------------------------------
namespace Server.Searches {
//--------------------------------------------------------------------------------------------------
public class SpiralSearch
{
    private const int           MAXRANGE = 24;
    private int                 m_nEntries;
    private SearchSet[]         m_SearchSets;
//--------------------------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------------------------
    public SpiralSearch( )
    {
        m_SearchSets            = new SearchSet[MAXRANGE+1];

        GenerateSearchSets();
    }
//--------------------------------------------------------------------------------------------------
//  Spiral search
//--------------------------------------------------------------------------------------------------
    public bool SearchSpiral( 
        Map                 map, 
        int                 start,
        SearchDirection     direction,
        bool                randomStart,
        Tour                tour 
        )
    {
        int                 incr = ( direction == SearchDirection.Outwards ? 1 : -1 );
        bool                firstpass = true;

        for( int i=start;;i+=incr)
        {
            if( i < 0 || i > MAXRANGE ) break;

            SearchSet           ss      = m_SearchSets[i];
            List<SpiralEntry>   entries = ss.Entries;

            int startpos;
            
            if( randomStart && firstpass ) 
            { 
                startpos = Utility.RandomMinMax( 0, entries.Count ); 
            }
            else
            { 
                startpos = ( direction == SearchDirection.Outwards ? 0 : entries.Count - 1 );
            }

            for( int pos = startpos;;pos+=incr )
            {
                if( pos < 0 || pos > entries.Count-1 ) break;

                SpiralEntry entry = entries[pos];

                if( tour( map, entry.X, entry.Y ) ) return true;
            }

            firstpass = false;
        }

        return false;
    }
//--------------------------------------------------------------------------------------------------
//  Create/sort search sets according to radii
//--------------------------------------------------------------------------------------------------
    private void GenerateSearchSets()
    {
        foreach( int[] point in m_SpiralPattern )
        {
            SpiralEntry entry       = new SpiralEntry( point[0], point[1] );
            int         range       = entry.Distance;

            SearchSet   searchSet;

            if( m_SearchSets[range] == null )
            {
                searchSet = new SearchSet( range );
                m_SearchSets[range] = searchSet;
            }
            else
            {
                searchSet = m_SearchSets[range];
            }

            searchSet.AddEntry( entry );
            m_nEntries++;
        }
    }
//--------------------------------------------------------------------------------------------------
//  Debugging
//--------------------------------------------------------------------------------------------------
    public void DumpSearchSets( )
    {
        Console.WriteLine("nEntries = "+m_nEntries);
        Console.WriteLine("mSearchSets.Length = "+m_SearchSets.Length);
        foreach( SearchSet ss in m_SearchSets )
        {
            Console.WriteLine( "Range = "+ss.Range );
            Console.WriteLine( "    "+ss );
            Console.WriteLine( );
        }
    }
//--------------------------------------------------------------------------------------------------
//  Debugging
//--------------------------------------------------------------------------------------------------
    public override string ToString( )
    {
        string str = "[ ";

        foreach( SearchSet ss in m_SearchSets )
            str += ss.ToString() + ", ";

            str += " ]";
        
        return str;
    }
//--------------------------------------------------------------------------------------------------
//  SearchSet -- all entry sets, sorted by THETA, are held in octants in the search area
//--------------------------------------------------------------------------------------------------
    public class SearchSet
    {
        private     int                    m_Range;
        private     List<SpiralEntry>      m_Entries;
        public      int                    Range{ get { return m_Range; } }
        public      List<SpiralEntry>      Entries { get { return m_Entries; } }

        public SearchSet( int range )
        {
            m_Range = range;
            m_Entries = new List<SpiralEntry>();
        }

        public override string ToString( )
        {
            string str = "[ ";

            foreach( SpiralEntry entry in m_Entries )
                str += entry.ToString() + ", ";

            str += " ]";
            
            return str;
        }

        public void AddEntry( SpiralEntry entry )
        {
            m_Entries.Add( entry );
        }
    }
//--------------------------------------------------------------------------------------------------
//  CirclEntry: just a place to store
//--------------------------------------------------------------------------------------------------
    public class SpiralEntry :  Entry
    {
        public SpiralEntry( int x, int y ) : base ( x, y, DistanceFunction.Linear )
        {
        }

        public override string ToString( )
        {
            return string.Format("{0}:{1}", X, Y);
        }
    }
//--------------------------------------------------------------------------------------------------
//  Data
//--------------------------------------------------------------------------------------------------
internal static int[][] m_SpiralPattern = new int[][]
{
              //  x    y
    new int[] {   0,   0 },
    new int[] {   0,   1 },
    new int[] {   1,   1 },
    new int[] {   1,   0 },
    new int[] {   1,  -1 },
    new int[] {   0,  -1 },
    new int[] {  -1,  -1 },
    new int[] {  -1,   0 },
    new int[] {  -1,   1 },
    new int[] {  -1,   2 },
    new int[] {   0,   2 },
    new int[] {   1,   2 },
    new int[] {   2,   2 },
    new int[] {   2,   1 },
    new int[] {   2,   0 },
    new int[] {   2,  -1 },
    new int[] {   2,  -2 },
    new int[] {   1,  -2 },
    new int[] {   0,  -2 },
    new int[] {  -1,  -2 },
    new int[] {  -2,  -2 },
    new int[] {  -2,  -1 },
    new int[] {  -2,   0 },
    new int[] {  -2,   1 },
    new int[] {  -2,   2 },
    new int[] {  -2,   3 },
    new int[] {  -1,   3 },
    new int[] {   0,   3 },
    new int[] {   1,   3 },
    new int[] {   2,   3 },
    new int[] {   3,   3 },
    new int[] {   3,   2 },
    new int[] {   3,   1 },
    new int[] {   3,   0 },
    new int[] {   3,  -1 },
    new int[] {   3,  -2 },
    new int[] {   3,  -3 },
    new int[] {   2,  -3 },
    new int[] {   1,  -3 },
    new int[] {   0,  -3 },
    new int[] {  -1,  -3 },
    new int[] {  -2,  -3 },
    new int[] {  -3,  -3 },
    new int[] {  -3,  -2 },
    new int[] {  -3,  -1 },
    new int[] {  -3,   0 },
    new int[] {  -3,   1 },
    new int[] {  -3,   2 },
    new int[] {  -3,   3 },
    new int[] {  -3,   4 },
    new int[] {  -2,   4 },
    new int[] {  -1,   4 },
    new int[] {   0,   4 },
    new int[] {   1,   4 },
    new int[] {   2,   4 },
    new int[] {   3,   4 },
    new int[] {   4,   4 },
    new int[] {   4,   3 },
    new int[] {   4,   2 },
    new int[] {   4,   1 },
    new int[] {   4,   0 },
    new int[] {   4,  -1 },
    new int[] {   4,  -2 },
    new int[] {   4,  -3 },
    new int[] {   4,  -4 },
    new int[] {   3,  -4 },
    new int[] {   2,  -4 },
    new int[] {   1,  -4 },
    new int[] {   0,  -4 },
    new int[] {  -1,  -4 },
    new int[] {  -2,  -4 },
    new int[] {  -3,  -4 },
    new int[] {  -4,  -4 },
    new int[] {  -4,  -3 },
    new int[] {  -4,  -2 },
    new int[] {  -4,  -1 },
    new int[] {  -4,   0 },
    new int[] {  -4,   1 },
    new int[] {  -4,   2 },
    new int[] {  -4,   3 },
    new int[] {  -4,   4 },
    new int[] {  -4,   5 },
    new int[] {  -3,   5 },
    new int[] {  -2,   5 },
    new int[] {  -1,   5 },
    new int[] {   0,   5 },
    new int[] {   1,   5 },
    new int[] {   2,   5 },
    new int[] {   3,   5 },
    new int[] {   4,   5 },
    new int[] {   5,   5 },
    new int[] {   5,   4 },
    new int[] {   5,   3 },
    new int[] {   5,   2 },
    new int[] {   5,   1 },
    new int[] {   5,   0 },
    new int[] {   5,  -1 },
    new int[] {   5,  -2 },
    new int[] {   5,  -3 },
    new int[] {   5,  -4 },
    new int[] {   5,  -5 },
    new int[] {   4,  -5 },
    new int[] {   3,  -5 },
    new int[] {   2,  -5 },
    new int[] {   1,  -5 },
    new int[] {   0,  -5 },
    new int[] {  -1,  -5 },
    new int[] {  -2,  -5 },
    new int[] {  -3,  -5 },
    new int[] {  -4,  -5 },
    new int[] {  -5,  -5 },
    new int[] {  -5,  -4 },
    new int[] {  -5,  -3 },
    new int[] {  -5,  -2 },
    new int[] {  -5,  -1 },
    new int[] {  -5,   0 },
    new int[] {  -5,   1 },
    new int[] {  -5,   2 },
    new int[] {  -5,   3 },
    new int[] {  -5,   4 },
    new int[] {  -5,   5 },
    new int[] {  -5,   6 },
    new int[] {  -4,   6 },
    new int[] {  -3,   6 },
    new int[] {  -2,   6 },
    new int[] {  -1,   6 },
    new int[] {   0,   6 },
    new int[] {   1,   6 },
    new int[] {   2,   6 },
    new int[] {   3,   6 },
    new int[] {   4,   6 },
    new int[] {   5,   6 },
    new int[] {   6,   6 },
    new int[] {   6,   5 },
    new int[] {   6,   4 },
    new int[] {   6,   3 },
    new int[] {   6,   2 },
    new int[] {   6,   1 },
    new int[] {   6,   0 },
    new int[] {   6,  -1 },
    new int[] {   6,  -2 },
    new int[] {   6,  -3 },
    new int[] {   6,  -4 },
    new int[] {   6,  -5 },
    new int[] {   6,  -6 },
    new int[] {   5,  -6 },
    new int[] {   4,  -6 },
    new int[] {   3,  -6 },
    new int[] {   2,  -6 },
    new int[] {   1,  -6 },
    new int[] {   0,  -6 },
    new int[] {  -1,  -6 },
    new int[] {  -2,  -6 },
    new int[] {  -3,  -6 },
    new int[] {  -4,  -6 },
    new int[] {  -5,  -6 },
    new int[] {  -6,  -6 },
    new int[] {  -6,  -5 },
    new int[] {  -6,  -4 },
    new int[] {  -6,  -3 },
    new int[] {  -6,  -2 },
    new int[] {  -6,  -1 },
    new int[] {  -6,   0 },
    new int[] {  -6,   1 },
    new int[] {  -6,   2 },
    new int[] {  -6,   3 },
    new int[] {  -6,   4 },
    new int[] {  -6,   5 },
    new int[] {  -6,   6 },
    new int[] {  -6,   7 },
    new int[] {  -5,   7 },
    new int[] {  -4,   7 },
    new int[] {  -3,   7 },
    new int[] {  -2,   7 },
    new int[] {  -1,   7 },
    new int[] {   0,   7 },
    new int[] {   1,   7 },
    new int[] {   2,   7 },
    new int[] {   3,   7 },
    new int[] {   4,   7 },
    new int[] {   5,   7 },
    new int[] {   6,   7 },
    new int[] {   7,   7 },
    new int[] {   7,   6 },
    new int[] {   7,   5 },
    new int[] {   7,   4 },
    new int[] {   7,   3 },
    new int[] {   7,   2 },
    new int[] {   7,   1 },
    new int[] {   7,   0 },
    new int[] {   7,  -1 },
    new int[] {   7,  -2 },
    new int[] {   7,  -3 },
    new int[] {   7,  -4 },
    new int[] {   7,  -5 },
    new int[] {   7,  -6 },
    new int[] {   7,  -7 },
    new int[] {   6,  -7 },
    new int[] {   5,  -7 },
    new int[] {   4,  -7 },
    new int[] {   3,  -7 },
    new int[] {   2,  -7 },
    new int[] {   1,  -7 },
    new int[] {   0,  -7 },
    new int[] {  -1,  -7 },
    new int[] {  -2,  -7 },
    new int[] {  -3,  -7 },
    new int[] {  -4,  -7 },
    new int[] {  -5,  -7 },
    new int[] {  -6,  -7 },
    new int[] {  -7,  -7 },
    new int[] {  -7,  -6 },
    new int[] {  -7,  -5 },
    new int[] {  -7,  -4 },
    new int[] {  -7,  -3 },
    new int[] {  -7,  -2 },
    new int[] {  -7,  -1 },
    new int[] {  -7,   0 },
    new int[] {  -7,   1 },
    new int[] {  -7,   2 },
    new int[] {  -7,   3 },
    new int[] {  -7,   4 },
    new int[] {  -7,   5 },
    new int[] {  -7,   6 },
    new int[] {  -7,   7 },
    new int[] {  -7,   8 },
    new int[] {  -6,   8 },
    new int[] {  -5,   8 },
    new int[] {  -4,   8 },
    new int[] {  -3,   8 },
    new int[] {  -2,   8 },
    new int[] {  -1,   8 },
    new int[] {   0,   8 },
    new int[] {   1,   8 },
    new int[] {   2,   8 },
    new int[] {   3,   8 },
    new int[] {   4,   8 },
    new int[] {   5,   8 },
    new int[] {   6,   8 },
    new int[] {   7,   8 },
    new int[] {   8,   8 },
    new int[] {   8,   7 },
    new int[] {   8,   6 },
    new int[] {   8,   5 },
    new int[] {   8,   4 },
    new int[] {   8,   3 },
    new int[] {   8,   2 },
    new int[] {   8,   1 },
    new int[] {   8,   0 },
    new int[] {   8,  -1 },
    new int[] {   8,  -2 },
    new int[] {   8,  -3 },
    new int[] {   8,  -4 },
    new int[] {   8,  -5 },
    new int[] {   8,  -6 },
    new int[] {   8,  -7 },
    new int[] {   8,  -8 },
    new int[] {   7,  -8 },
    new int[] {   6,  -8 },
    new int[] {   5,  -8 },
    new int[] {   4,  -8 },
    new int[] {   3,  -8 },
    new int[] {   2,  -8 },
    new int[] {   1,  -8 },
    new int[] {   0,  -8 },
    new int[] {  -1,  -8 },
    new int[] {  -2,  -8 },
    new int[] {  -3,  -8 },
    new int[] {  -4,  -8 },
    new int[] {  -5,  -8 },
    new int[] {  -6,  -8 },
    new int[] {  -7,  -8 },
    new int[] {  -8,  -8 },
    new int[] {  -8,  -7 },
    new int[] {  -8,  -6 },
    new int[] {  -8,  -5 },
    new int[] {  -8,  -4 },
    new int[] {  -8,  -3 },
    new int[] {  -8,  -2 },
    new int[] {  -8,  -1 },
    new int[] {  -8,   0 },
    new int[] {  -8,   1 },
    new int[] {  -8,   2 },
    new int[] {  -8,   3 },
    new int[] {  -8,   4 },
    new int[] {  -8,   5 },
    new int[] {  -8,   6 },
    new int[] {  -8,   7 },
    new int[] {  -8,   8 },
    new int[] {  -8,   9 },
    new int[] {  -7,   9 },
    new int[] {  -6,   9 },
    new int[] {  -5,   9 },
    new int[] {  -4,   9 },
    new int[] {  -3,   9 },
    new int[] {  -2,   9 },
    new int[] {  -1,   9 },
    new int[] {   0,   9 },
    new int[] {   1,   9 },
    new int[] {   2,   9 },
    new int[] {   3,   9 },
    new int[] {   4,   9 },
    new int[] {   5,   9 },
    new int[] {   6,   9 },
    new int[] {   7,   9 },
    new int[] {   8,   9 },
    new int[] {   9,   9 },
    new int[] {   9,   8 },
    new int[] {   9,   7 },
    new int[] {   9,   6 },
    new int[] {   9,   5 },
    new int[] {   9,   4 },
    new int[] {   9,   3 },
    new int[] {   9,   2 },
    new int[] {   9,   1 },
    new int[] {   9,   0 },
    new int[] {   9,  -1 },
    new int[] {   9,  -2 },
    new int[] {   9,  -3 },
    new int[] {   9,  -4 },
    new int[] {   9,  -5 },
    new int[] {   9,  -6 },
    new int[] {   9,  -7 },
    new int[] {   9,  -8 },
    new int[] {   9,  -9 },
    new int[] {   8,  -9 },
    new int[] {   7,  -9 },
    new int[] {   6,  -9 },
    new int[] {   5,  -9 },
    new int[] {   4,  -9 },
    new int[] {   3,  -9 },
    new int[] {   2,  -9 },
    new int[] {   1,  -9 },
    new int[] {   0,  -9 },
    new int[] {  -1,  -9 },
    new int[] {  -2,  -9 },
    new int[] {  -3,  -9 },
    new int[] {  -4,  -9 },
    new int[] {  -5,  -9 },
    new int[] {  -6,  -9 },
    new int[] {  -7,  -9 },
    new int[] {  -8,  -9 },
    new int[] {  -9,  -9 },
    new int[] {  -9,  -8 },
    new int[] {  -9,  -7 },
    new int[] {  -9,  -6 },
    new int[] {  -9,  -5 },
    new int[] {  -9,  -4 },
    new int[] {  -9,  -3 },
    new int[] {  -9,  -2 },
    new int[] {  -9,  -1 },
    new int[] {  -9,   0 },
    new int[] {  -9,   1 },
    new int[] {  -9,   2 },
    new int[] {  -9,   3 },
    new int[] {  -9,   4 },
    new int[] {  -9,   5 },
    new int[] {  -9,   6 },
    new int[] {  -9,   7 },
    new int[] {  -9,   8 },
    new int[] {  -9,   9 },
    new int[] {  -9,  10 },
    new int[] {  -8,  10 },
    new int[] {  -7,  10 },
    new int[] {  -6,  10 },
    new int[] {  -5,  10 },
    new int[] {  -4,  10 },
    new int[] {  -3,  10 },
    new int[] {  -2,  10 },
    new int[] {  -1,  10 },
    new int[] {   0,  10 },
    new int[] {   1,  10 },
    new int[] {   2,  10 },
    new int[] {   3,  10 },
    new int[] {   4,  10 },
    new int[] {   5,  10 },
    new int[] {   6,  10 },
    new int[] {   7,  10 },
    new int[] {   8,  10 },
    new int[] {   9,  10 },
    new int[] {  10,  10 },
    new int[] {  10,   9 },
    new int[] {  10,   8 },
    new int[] {  10,   7 },
    new int[] {  10,   6 },
    new int[] {  10,   5 },
    new int[] {  10,   4 },
    new int[] {  10,   3 },
    new int[] {  10,   2 },
    new int[] {  10,   1 },
    new int[] {  10,   0 },
    new int[] {  10,  -1 },
    new int[] {  10,  -2 },
    new int[] {  10,  -3 },
    new int[] {  10,  -4 },
    new int[] {  10,  -5 },
    new int[] {  10,  -6 },
    new int[] {  10,  -7 },
    new int[] {  10,  -8 },
    new int[] {  10,  -9 },
    new int[] {  10, -10 },
    new int[] {   9, -10 },
    new int[] {   8, -10 },
    new int[] {   7, -10 },
    new int[] {   6, -10 },
    new int[] {   5, -10 },
    new int[] {   4, -10 },
    new int[] {   3, -10 },
    new int[] {   2, -10 },
    new int[] {   1, -10 },
    new int[] {   0, -10 },
    new int[] {  -1, -10 },
    new int[] {  -2, -10 },
    new int[] {  -3, -10 },
    new int[] {  -4, -10 },
    new int[] {  -5, -10 },
    new int[] {  -6, -10 },
    new int[] {  -7, -10 },
    new int[] {  -8, -10 },
    new int[] {  -9, -10 },
    new int[] { -10, -10 },
    new int[] { -10,  -9 },
    new int[] { -10,  -8 },
    new int[] { -10,  -7 },
    new int[] { -10,  -6 },
    new int[] { -10,  -5 },
    new int[] { -10,  -4 },
    new int[] { -10,  -3 },
    new int[] { -10,  -2 },
    new int[] { -10,  -1 },
    new int[] { -10,   0 },
    new int[] { -10,   1 },
    new int[] { -10,   2 },
    new int[] { -10,   3 },
    new int[] { -10,   4 },
    new int[] { -10,   5 },
    new int[] { -10,   6 },
    new int[] { -10,   7 },
    new int[] { -10,   8 },
    new int[] { -10,   9 },
    new int[] { -10,  10 },
    new int[] { -10,  11 },
    new int[] {  -9,  11 },
    new int[] {  -8,  11 },
    new int[] {  -7,  11 },
    new int[] {  -6,  11 },
    new int[] {  -5,  11 },
    new int[] {  -4,  11 },
    new int[] {  -3,  11 },
    new int[] {  -2,  11 },
    new int[] {  -1,  11 },
    new int[] {   0,  11 },
    new int[] {   1,  11 },
    new int[] {   2,  11 },
    new int[] {   3,  11 },
    new int[] {   4,  11 },
    new int[] {   5,  11 },
    new int[] {   6,  11 },
    new int[] {   7,  11 },
    new int[] {   8,  11 },
    new int[] {   9,  11 },
    new int[] {  10,  11 },
    new int[] {  11,  11 },
    new int[] {  11,  10 },
    new int[] {  11,   9 },
    new int[] {  11,   8 },
    new int[] {  11,   7 },
    new int[] {  11,   6 },
    new int[] {  11,   5 },
    new int[] {  11,   4 },
    new int[] {  11,   3 },
    new int[] {  11,   2 },
    new int[] {  11,   1 },
    new int[] {  11,   0 },
    new int[] {  11,  -1 },
    new int[] {  11,  -2 },
    new int[] {  11,  -3 },
    new int[] {  11,  -4 },
    new int[] {  11,  -5 },
    new int[] {  11,  -6 },
    new int[] {  11,  -7 },
    new int[] {  11,  -8 },
    new int[] {  11,  -9 },
    new int[] {  11, -10 },
    new int[] {  11, -11 },
    new int[] {  10, -11 },
    new int[] {   9, -11 },
    new int[] {   8, -11 },
    new int[] {   7, -11 },
    new int[] {   6, -11 },
    new int[] {   5, -11 },
    new int[] {   4, -11 },
    new int[] {   3, -11 },
    new int[] {   2, -11 },
    new int[] {   1, -11 },
    new int[] {   0, -11 },
    new int[] {  -1, -11 },
    new int[] {  -2, -11 },
    new int[] {  -3, -11 },
    new int[] {  -4, -11 },
    new int[] {  -5, -11 },
    new int[] {  -6, -11 },
    new int[] {  -7, -11 },
    new int[] {  -8, -11 },
    new int[] {  -9, -11 },
    new int[] { -10, -11 },
    new int[] { -11, -11 },
    new int[] { -11, -10 },
    new int[] { -11,  -9 },
    new int[] { -11,  -8 },
    new int[] { -11,  -7 },
    new int[] { -11,  -6 },
    new int[] { -11,  -5 },
    new int[] { -11,  -4 },
    new int[] { -11,  -3 },
    new int[] { -11,  -2 },
    new int[] { -11,  -1 },
    new int[] { -11,   0 },
    new int[] { -11,   1 },
    new int[] { -11,   2 },
    new int[] { -11,   3 },
    new int[] { -11,   4 },
    new int[] { -11,   5 },
    new int[] { -11,   6 },
    new int[] { -11,   7 },
    new int[] { -11,   8 },
    new int[] { -11,   9 },
    new int[] { -11,  10 },
    new int[] { -11,  11 },
    new int[] { -11,  12 },
    new int[] { -10,  12 },
    new int[] {  -9,  12 },
    new int[] {  -8,  12 },
    new int[] {  -7,  12 },
    new int[] {  -6,  12 },
    new int[] {  -5,  12 },
    new int[] {  -4,  12 },
    new int[] {  -3,  12 },
    new int[] {  -2,  12 },
    new int[] {  -1,  12 },
    new int[] {   0,  12 },
    new int[] {   1,  12 },
    new int[] {   2,  12 },
    new int[] {   3,  12 },
    new int[] {   4,  12 },
    new int[] {   5,  12 },
    new int[] {   6,  12 },
    new int[] {   7,  12 },
    new int[] {   8,  12 },
    new int[] {   9,  12 },
    new int[] {  10,  12 },
    new int[] {  11,  12 },
    new int[] {  12,  12 },
    new int[] {  12,  11 },
    new int[] {  12,  10 },
    new int[] {  12,   9 },
    new int[] {  12,   8 },
    new int[] {  12,   7 },
    new int[] {  12,   6 },
    new int[] {  12,   5 },
    new int[] {  12,   4 },
    new int[] {  12,   3 },
    new int[] {  12,   2 },
    new int[] {  12,   1 },
    new int[] {  12,   0 },
    new int[] {  12,  -1 },
    new int[] {  12,  -2 },
    new int[] {  12,  -3 },
    new int[] {  12,  -4 },
    new int[] {  12,  -5 },
    new int[] {  12,  -6 },
    new int[] {  12,  -7 },
    new int[] {  12,  -8 },
    new int[] {  12,  -9 },
    new int[] {  12, -10 },
    new int[] {  12, -11 },
    new int[] {  12, -12 },
    new int[] {  11, -12 },
    new int[] {  10, -12 },
    new int[] {   9, -12 },
    new int[] {   8, -12 },
    new int[] {   7, -12 },
    new int[] {   6, -12 },
    new int[] {   5, -12 },
    new int[] {   4, -12 },
    new int[] {   3, -12 },
    new int[] {   2, -12 },
    new int[] {   1, -12 },
    new int[] {   0, -12 },
    new int[] {  -1, -12 },
    new int[] {  -2, -12 },
    new int[] {  -3, -12 },
    new int[] {  -4, -12 },
    new int[] {  -5, -12 },
    new int[] {  -6, -12 },
    new int[] {  -7, -12 },
    new int[] {  -8, -12 },
    new int[] {  -9, -12 },
    new int[] { -10, -12 },
    new int[] { -11, -12 },
    new int[] { -12, -12 },
    new int[] { -12, -11 },
    new int[] { -12, -10 },
    new int[] { -12,  -9 },
    new int[] { -12,  -8 },
    new int[] { -12,  -7 },
    new int[] { -12,  -6 },
    new int[] { -12,  -5 },
    new int[] { -12,  -4 },
    new int[] { -12,  -3 },
    new int[] { -12,  -2 },
    new int[] { -12,  -1 },
    new int[] { -12,   0 },
    new int[] { -12,   1 },
    new int[] { -12,   2 },
    new int[] { -12,   3 },
    new int[] { -12,   4 },
    new int[] { -12,   5 },
    new int[] { -12,   6 },
    new int[] { -12,   7 },
    new int[] { -12,   8 },
    new int[] { -12,   9 },
    new int[] { -12,  10 },
    new int[] { -12,  11 },
    new int[] { -12,  12 },
    new int[] { -12,  13 },
    new int[] { -11,  13 },
    new int[] { -10,  13 },
    new int[] {  -9,  13 },
    new int[] {  -8,  13 },
    new int[] {  -7,  13 },
    new int[] {  -6,  13 },
    new int[] {  -5,  13 },
    new int[] {  -4,  13 },
    new int[] {  -3,  13 },
    new int[] {  -2,  13 },
    new int[] {  -1,  13 },
    new int[] {   0,  13 },
    new int[] {   1,  13 },
    new int[] {   2,  13 },
    new int[] {   3,  13 },
    new int[] {   4,  13 },
    new int[] {   5,  13 },
    new int[] {   6,  13 },
    new int[] {   7,  13 },
    new int[] {   8,  13 },
    new int[] {   9,  13 },
    new int[] {  10,  13 },
    new int[] {  11,  13 },
    new int[] {  12,  13 },
    new int[] {  13,  13 },
    new int[] {  13,  12 },
    new int[] {  13,  11 },
    new int[] {  13,  10 },
    new int[] {  13,   9 },
    new int[] {  13,   8 },
    new int[] {  13,   7 },
    new int[] {  13,   6 },
    new int[] {  13,   5 },
    new int[] {  13,   4 },
    new int[] {  13,   3 },
    new int[] {  13,   2 },
    new int[] {  13,   1 },
    new int[] {  13,   0 },
    new int[] {  13,  -1 },
    new int[] {  13,  -2 },
    new int[] {  13,  -3 },
    new int[] {  13,  -4 },
    new int[] {  13,  -5 },
    new int[] {  13,  -6 },
    new int[] {  13,  -7 },
    new int[] {  13,  -8 },
    new int[] {  13,  -9 },
    new int[] {  13, -10 },
    new int[] {  13, -11 },
    new int[] {  13, -12 },
    new int[] {  13, -13 },
    new int[] {  12, -13 },
    new int[] {  11, -13 },
    new int[] {  10, -13 },
    new int[] {   9, -13 },
    new int[] {   8, -13 },
    new int[] {   7, -13 },
    new int[] {   6, -13 },
    new int[] {   5, -13 },
    new int[] {   4, -13 },
    new int[] {   3, -13 },
    new int[] {   2, -13 },
    new int[] {   1, -13 },
    new int[] {   0, -13 },
    new int[] {  -1, -13 },
    new int[] {  -2, -13 },
    new int[] {  -3, -13 },
    new int[] {  -4, -13 },
    new int[] {  -5, -13 },
    new int[] {  -6, -13 },
    new int[] {  -7, -13 },
    new int[] {  -8, -13 },
    new int[] {  -9, -13 },
    new int[] { -10, -13 },
    new int[] { -11, -13 },
    new int[] { -12, -13 },
    new int[] { -13, -13 },
    new int[] { -13, -12 },
    new int[] { -13, -11 },
    new int[] { -13, -10 },
    new int[] { -13,  -9 },
    new int[] { -13,  -8 },
    new int[] { -13,  -7 },
    new int[] { -13,  -6 },
    new int[] { -13,  -5 },
    new int[] { -13,  -4 },
    new int[] { -13,  -3 },
    new int[] { -13,  -2 },
    new int[] { -13,  -1 },
    new int[] { -13,   0 },
    new int[] { -13,   1 },
    new int[] { -13,   2 },
    new int[] { -13,   3 },
    new int[] { -13,   4 },
    new int[] { -13,   5 },
    new int[] { -13,   6 },
    new int[] { -13,   7 },
    new int[] { -13,   8 },
    new int[] { -13,   9 },
    new int[] { -13,  10 },
    new int[] { -13,  11 },
    new int[] { -13,  12 },
    new int[] { -13,  13 },
    new int[] { -13,  14 },
    new int[] { -12,  14 },
    new int[] { -11,  14 },
    new int[] { -10,  14 },
    new int[] {  -9,  14 },
    new int[] {  -8,  14 },
    new int[] {  -7,  14 },
    new int[] {  -6,  14 },
    new int[] {  -5,  14 },
    new int[] {  -4,  14 },
    new int[] {  -3,  14 },
    new int[] {  -2,  14 },
    new int[] {  -1,  14 },
    new int[] {   0,  14 },
    new int[] {   1,  14 },
    new int[] {   2,  14 },
    new int[] {   3,  14 },
    new int[] {   4,  14 },
    new int[] {   5,  14 },
    new int[] {   6,  14 },
    new int[] {   7,  14 },
    new int[] {   8,  14 },
    new int[] {   9,  14 },
    new int[] {  10,  14 },
    new int[] {  11,  14 },
    new int[] {  12,  14 },
    new int[] {  13,  14 },
    new int[] {  14,  14 },
    new int[] {  14,  13 },
    new int[] {  14,  12 },
    new int[] {  14,  11 },
    new int[] {  14,  10 },
    new int[] {  14,   9 },
    new int[] {  14,   8 },
    new int[] {  14,   7 },
    new int[] {  14,   6 },
    new int[] {  14,   5 },
    new int[] {  14,   4 },
    new int[] {  14,   3 },
    new int[] {  14,   2 },
    new int[] {  14,   1 },
    new int[] {  14,   0 },
    new int[] {  14,  -1 },
    new int[] {  14,  -2 },
    new int[] {  14,  -3 },
    new int[] {  14,  -4 },
    new int[] {  14,  -5 },
    new int[] {  14,  -6 },
    new int[] {  14,  -7 },
    new int[] {  14,  -8 },
    new int[] {  14,  -9 },
    new int[] {  14, -10 },
    new int[] {  14, -11 },
    new int[] {  14, -12 },
    new int[] {  14, -13 },
    new int[] {  14, -14 },
    new int[] {  13, -14 },
    new int[] {  12, -14 },
    new int[] {  11, -14 },
    new int[] {  10, -14 },
    new int[] {   9, -14 },
    new int[] {   8, -14 },
    new int[] {   7, -14 },
    new int[] {   6, -14 },
    new int[] {   5, -14 },
    new int[] {   4, -14 },
    new int[] {   3, -14 },
    new int[] {   2, -14 },
    new int[] {   1, -14 },
    new int[] {   0, -14 },
    new int[] {  -1, -14 },
    new int[] {  -2, -14 },
    new int[] {  -3, -14 },
    new int[] {  -4, -14 },
    new int[] {  -5, -14 },
    new int[] {  -6, -14 },
    new int[] {  -7, -14 },
    new int[] {  -8, -14 },
    new int[] {  -9, -14 },
    new int[] { -10, -14 },
    new int[] { -11, -14 },
    new int[] { -12, -14 },
    new int[] { -13, -14 },
    new int[] { -14, -14 },
    new int[] { -14, -13 },
    new int[] { -14, -12 },
    new int[] { -14, -11 },
    new int[] { -14, -10 },
    new int[] { -14,  -9 },
    new int[] { -14,  -8 },
    new int[] { -14,  -7 },
    new int[] { -14,  -6 },
    new int[] { -14,  -5 },
    new int[] { -14,  -4 },
    new int[] { -14,  -3 },
    new int[] { -14,  -2 },
    new int[] { -14,  -1 },
    new int[] { -14,   0 },
    new int[] { -14,   1 },
    new int[] { -14,   2 },
    new int[] { -14,   3 },
    new int[] { -14,   4 },
    new int[] { -14,   5 },
    new int[] { -14,   6 },
    new int[] { -14,   7 },
    new int[] { -14,   8 },
    new int[] { -14,   9 },
    new int[] { -14,  10 },
    new int[] { -14,  11 },
    new int[] { -14,  12 },
    new int[] { -14,  13 },
    new int[] { -14,  14 },
    new int[] { -14,  15 },
    new int[] { -13,  15 },
    new int[] { -12,  15 },
    new int[] { -11,  15 },
    new int[] { -10,  15 },
    new int[] {  -9,  15 },
    new int[] {  -8,  15 },
    new int[] {  -7,  15 },
    new int[] {  -6,  15 },
    new int[] {  -5,  15 },
    new int[] {  -4,  15 },
    new int[] {  -3,  15 },
    new int[] {  -2,  15 },
    new int[] {  -1,  15 },
    new int[] {   0,  15 },
    new int[] {   1,  15 },
    new int[] {   2,  15 },
    new int[] {   3,  15 },
    new int[] {   4,  15 },
    new int[] {   5,  15 },
    new int[] {   6,  15 },
    new int[] {   7,  15 },
    new int[] {   8,  15 },
    new int[] {   9,  15 },
    new int[] {  10,  15 },
    new int[] {  11,  15 },
    new int[] {  12,  15 },
    new int[] {  13,  15 },
    new int[] {  14,  15 },
    new int[] {  15,  15 },
    new int[] {  15,  14 },
    new int[] {  15,  13 },
    new int[] {  15,  12 },
    new int[] {  15,  11 },
    new int[] {  15,  10 },
    new int[] {  15,   9 },
    new int[] {  15,   8 },
    new int[] {  15,   7 },
    new int[] {  15,   6 },
    new int[] {  15,   5 },
    new int[] {  15,   4 },
    new int[] {  15,   3 },
    new int[] {  15,   2 },
    new int[] {  15,   1 },
    new int[] {  15,   0 },
    new int[] {  15,  -1 },
    new int[] {  15,  -2 },
    new int[] {  15,  -3 },
    new int[] {  15,  -4 },
    new int[] {  15,  -5 },
    new int[] {  15,  -6 },
    new int[] {  15,  -7 },
    new int[] {  15,  -8 },
    new int[] {  15,  -9 },
    new int[] {  15, -10 },
    new int[] {  15, -11 },
    new int[] {  15, -12 },
    new int[] {  15, -13 },
    new int[] {  15, -14 },
    new int[] {  15, -15 },
    new int[] {  14, -15 },
    new int[] {  13, -15 },
    new int[] {  12, -15 },
    new int[] {  11, -15 },
    new int[] {  10, -15 },
    new int[] {   9, -15 },
    new int[] {   8, -15 },
    new int[] {   7, -15 },
    new int[] {   6, -15 },
    new int[] {   5, -15 },
    new int[] {   4, -15 },
    new int[] {   3, -15 },
    new int[] {   2, -15 },
    new int[] {   1, -15 },
    new int[] {   0, -15 },
    new int[] {  -1, -15 },
    new int[] {  -2, -15 },
    new int[] {  -3, -15 },
    new int[] {  -4, -15 },
    new int[] {  -5, -15 },
    new int[] {  -6, -15 },
    new int[] {  -7, -15 },
    new int[] {  -8, -15 },
    new int[] {  -9, -15 },
    new int[] { -10, -15 },
    new int[] { -11, -15 },
    new int[] { -12, -15 },
    new int[] { -13, -15 },
    new int[] { -14, -15 },
    new int[] { -15, -15 },
    new int[] { -15, -14 },
    new int[] { -15, -13 },
    new int[] { -15, -12 },
    new int[] { -15, -11 },
    new int[] { -15, -10 },
    new int[] { -15,  -9 },
    new int[] { -15,  -8 },
    new int[] { -15,  -7 },
    new int[] { -15,  -6 },
    new int[] { -15,  -5 },
    new int[] { -15,  -4 },
    new int[] { -15,  -3 },
    new int[] { -15,  -2 },
    new int[] { -15,  -1 },
    new int[] { -15,   0 },
    new int[] { -15,   1 },
    new int[] { -15,   2 },
    new int[] { -15,   3 },
    new int[] { -15,   4 },
    new int[] { -15,   5 },
    new int[] { -15,   6 },
    new int[] { -15,   7 },
    new int[] { -15,   8 },
    new int[] { -15,   9 },
    new int[] { -15,  10 },
    new int[] { -15,  11 },
    new int[] { -15,  12 },
    new int[] { -15,  13 },
    new int[] { -15,  14 },
    new int[] { -15,  15 },
    new int[] { -15,  16 },
    new int[] { -14,  16 },
    new int[] { -13,  16 },
    new int[] { -12,  16 },
    new int[] { -11,  16 },
    new int[] { -10,  16 },
    new int[] {  -9,  16 },
    new int[] {  -8,  16 },
    new int[] {  -7,  16 },
    new int[] {  -6,  16 },
    new int[] {  -5,  16 },
    new int[] {  -4,  16 },
    new int[] {  -3,  16 },
    new int[] {  -2,  16 },
    new int[] {  -1,  16 },
    new int[] {   0,  16 },
    new int[] {   1,  16 },
    new int[] {   2,  16 },
    new int[] {   3,  16 },
    new int[] {   4,  16 },
    new int[] {   5,  16 },
    new int[] {   6,  16 },
    new int[] {   7,  16 },
    new int[] {   8,  16 },
    new int[] {   9,  16 },
    new int[] {  10,  16 },
    new int[] {  11,  16 },
    new int[] {  12,  16 },
    new int[] {  13,  16 },
    new int[] {  14,  16 },
    new int[] {  15,  16 },
    new int[] {  16,  16 },
    new int[] {  16,  15 },
    new int[] {  16,  14 },
    new int[] {  16,  13 },
    new int[] {  16,  12 },
    new int[] {  16,  11 },
    new int[] {  16,  10 },
    new int[] {  16,   9 },
    new int[] {  16,   8 },
    new int[] {  16,   7 },
    new int[] {  16,   6 },
    new int[] {  16,   5 },
    new int[] {  16,   4 },
    new int[] {  16,   3 },
    new int[] {  16,   2 },
    new int[] {  16,   1 },
    new int[] {  16,   0 },
    new int[] {  16,  -1 },
    new int[] {  16,  -2 },
    new int[] {  16,  -3 },
    new int[] {  16,  -4 },
    new int[] {  16,  -5 },
    new int[] {  16,  -6 },
    new int[] {  16,  -7 },
    new int[] {  16,  -8 },
    new int[] {  16,  -9 },
    new int[] {  16, -10 },
    new int[] {  16, -11 },
    new int[] {  16, -12 },
    new int[] {  16, -13 },
    new int[] {  16, -14 },
    new int[] {  16, -15 },
    new int[] {  16, -16 },
    new int[] {  15, -16 },
    new int[] {  14, -16 },
    new int[] {  13, -16 },
    new int[] {  12, -16 },
    new int[] {  11, -16 },
    new int[] {  10, -16 },
    new int[] {   9, -16 },
    new int[] {   8, -16 },
    new int[] {   7, -16 },
    new int[] {   6, -16 },
    new int[] {   5, -16 },
    new int[] {   4, -16 },
    new int[] {   3, -16 },
    new int[] {   2, -16 },
    new int[] {   1, -16 },
    new int[] {   0, -16 },
    new int[] {  -1, -16 },
    new int[] {  -2, -16 },
    new int[] {  -3, -16 },
    new int[] {  -4, -16 },
    new int[] {  -5, -16 },
    new int[] {  -6, -16 },
    new int[] {  -7, -16 },
    new int[] {  -8, -16 },
    new int[] {  -9, -16 },
    new int[] { -10, -16 },
    new int[] { -11, -16 },
    new int[] { -12, -16 },
    new int[] { -13, -16 },
    new int[] { -14, -16 },
    new int[] { -15, -16 },
    new int[] { -16, -16 },
    new int[] { -16, -15 },
    new int[] { -16, -14 },
    new int[] { -16, -13 },
    new int[] { -16, -12 },
    new int[] { -16, -11 },
    new int[] { -16, -10 },
    new int[] { -16,  -9 },
    new int[] { -16,  -8 },
    new int[] { -16,  -7 },
    new int[] { -16,  -6 },
    new int[] { -16,  -5 },
    new int[] { -16,  -4 },
    new int[] { -16,  -3 },
    new int[] { -16,  -2 },
    new int[] { -16,  -1 },
    new int[] { -16,   0 },
    new int[] { -16,   1 },
    new int[] { -16,   2 },
    new int[] { -16,   3 },
    new int[] { -16,   4 },
    new int[] { -16,   5 },
    new int[] { -16,   6 },
    new int[] { -16,   7 },
    new int[] { -16,   8 },
    new int[] { -16,   9 },
    new int[] { -16,  10 },
    new int[] { -16,  11 },
    new int[] { -16,  12 },
    new int[] { -16,  13 },
    new int[] { -16,  14 },
    new int[] { -16,  15 },
    new int[] { -16,  16 },
    new int[] { -16,  17 },
    new int[] { -15,  17 },
    new int[] { -14,  17 },
    new int[] { -13,  17 },
    new int[] { -12,  17 },
    new int[] { -11,  17 },
    new int[] { -10,  17 },
    new int[] {  -9,  17 },
    new int[] {  -8,  17 },
    new int[] {  -7,  17 },
    new int[] {  -6,  17 },
    new int[] {  -5,  17 },
    new int[] {  -4,  17 },
    new int[] {  -3,  17 },
    new int[] {  -2,  17 },
    new int[] {  -1,  17 },
    new int[] {   0,  17 },
    new int[] {   1,  17 },
    new int[] {   2,  17 },
    new int[] {   3,  17 },
    new int[] {   4,  17 },
    new int[] {   5,  17 },
    new int[] {   6,  17 },
    new int[] {   7,  17 },
    new int[] {   8,  17 },
    new int[] {   9,  17 },
    new int[] {  10,  17 },
    new int[] {  11,  17 },
    new int[] {  12,  17 },
    new int[] {  13,  17 },
    new int[] {  14,  17 },
    new int[] {  15,  17 },
    new int[] {  16,  17 },
    new int[] {  17,  17 },
    new int[] {  17,  16 },
    new int[] {  17,  15 },
    new int[] {  17,  14 },
    new int[] {  17,  13 },
    new int[] {  17,  12 },
    new int[] {  17,  11 },
    new int[] {  17,  10 },
    new int[] {  17,   9 },
    new int[] {  17,   8 },
    new int[] {  17,   7 },
    new int[] {  17,   6 },
    new int[] {  17,   5 },
    new int[] {  17,   4 },
    new int[] {  17,   3 },
    new int[] {  17,   2 },
    new int[] {  17,   1 },
    new int[] {  17,   0 },
    new int[] {  17,  -1 },
    new int[] {  17,  -2 },
    new int[] {  17,  -3 },
    new int[] {  17,  -4 },
    new int[] {  17,  -5 },
    new int[] {  17,  -6 },
    new int[] {  17,  -7 },
    new int[] {  17,  -8 },
    new int[] {  17,  -9 },
    new int[] {  17, -10 },
    new int[] {  17, -11 },
    new int[] {  17, -12 },
    new int[] {  17, -13 },
    new int[] {  17, -14 },
    new int[] {  17, -15 },
    new int[] {  17, -16 },
    new int[] {  17, -17 },
    new int[] {  16, -17 },
    new int[] {  15, -17 },
    new int[] {  14, -17 },
    new int[] {  13, -17 },
    new int[] {  12, -17 },
    new int[] {  11, -17 },
    new int[] {  10, -17 },
    new int[] {   9, -17 },
    new int[] {   8, -17 },
    new int[] {   7, -17 },
    new int[] {   6, -17 },
    new int[] {   5, -17 },
    new int[] {   4, -17 },
    new int[] {   3, -17 },
    new int[] {   2, -17 },
    new int[] {   1, -17 },
    new int[] {   0, -17 },
    new int[] {  -1, -17 },
    new int[] {  -2, -17 },
    new int[] {  -3, -17 },
    new int[] {  -4, -17 },
    new int[] {  -5, -17 },
    new int[] {  -6, -17 },
    new int[] {  -7, -17 },
    new int[] {  -8, -17 },
    new int[] {  -9, -17 },
    new int[] { -10, -17 },
    new int[] { -11, -17 },
    new int[] { -12, -17 },
    new int[] { -13, -17 },
    new int[] { -14, -17 },
    new int[] { -15, -17 },
    new int[] { -16, -17 },
    new int[] { -17, -17 },
    new int[] { -17, -16 },
    new int[] { -17, -15 },
    new int[] { -17, -14 },
    new int[] { -17, -13 },
    new int[] { -17, -12 },
    new int[] { -17, -11 },
    new int[] { -17, -10 },
    new int[] { -17,  -9 },
    new int[] { -17,  -8 },
    new int[] { -17,  -7 },
    new int[] { -17,  -6 },
    new int[] { -17,  -5 },
    new int[] { -17,  -4 },
    new int[] { -17,  -3 },
    new int[] { -17,  -2 },
    new int[] { -17,  -1 },
    new int[] { -17,   0 },
    new int[] { -17,   1 },
    new int[] { -17,   2 },
    new int[] { -17,   3 },
    new int[] { -17,   4 },
    new int[] { -17,   5 },
    new int[] { -17,   6 },
    new int[] { -17,   7 },
    new int[] { -17,   8 },
    new int[] { -17,   9 },
    new int[] { -17,  10 },
    new int[] { -17,  11 },
    new int[] { -17,  12 },
    new int[] { -17,  13 },
    new int[] { -17,  14 },
    new int[] { -17,  15 },
    new int[] { -17,  16 },
    new int[] { -17,  17 },
    new int[] { -17,  18 },
    new int[] { -16,  18 },
    new int[] { -15,  18 },
    new int[] { -14,  18 },
    new int[] { -13,  18 },
    new int[] { -12,  18 },
    new int[] { -11,  18 },
    new int[] { -10,  18 },
    new int[] {  -9,  18 },
    new int[] {  -8,  18 },
    new int[] {  -7,  18 },
    new int[] {  -6,  18 },
    new int[] {  -5,  18 },
    new int[] {  -4,  18 },
    new int[] {  -3,  18 },
    new int[] {  -2,  18 },
    new int[] {  -1,  18 },
    new int[] {   0,  18 },
    new int[] {   1,  18 },
    new int[] {   2,  18 },
    new int[] {   3,  18 },
    new int[] {   4,  18 },
    new int[] {   5,  18 },
    new int[] {   6,  18 },
    new int[] {   7,  18 },
    new int[] {   8,  18 },
    new int[] {   9,  18 },
    new int[] {  10,  18 },
    new int[] {  11,  18 },
    new int[] {  12,  18 },
    new int[] {  13,  18 },
    new int[] {  14,  18 },
    new int[] {  15,  18 },
    new int[] {  16,  18 },
    new int[] {  17,  18 },
    new int[] {  18,  18 },
    new int[] {  18,  17 },
    new int[] {  18,  16 },
    new int[] {  18,  15 },
    new int[] {  18,  14 },
    new int[] {  18,  13 },
    new int[] {  18,  12 },
    new int[] {  18,  11 },
    new int[] {  18,  10 },
    new int[] {  18,   9 },
    new int[] {  18,   8 },
    new int[] {  18,   7 },
    new int[] {  18,   6 },
    new int[] {  18,   5 },
    new int[] {  18,   4 },
    new int[] {  18,   3 },
    new int[] {  18,   2 },
    new int[] {  18,   1 },
    new int[] {  18,   0 },
    new int[] {  18,  -1 },
    new int[] {  18,  -2 },
    new int[] {  18,  -3 },
    new int[] {  18,  -4 },
    new int[] {  18,  -5 },
    new int[] {  18,  -6 },
    new int[] {  18,  -7 },
    new int[] {  18,  -8 },
    new int[] {  18,  -9 },
    new int[] {  18, -10 },
    new int[] {  18, -11 },
    new int[] {  18, -12 },
    new int[] {  18, -13 },
    new int[] {  18, -14 },
    new int[] {  18, -15 },
    new int[] {  18, -16 },
    new int[] {  18, -17 },
    new int[] {  18, -18 },
    new int[] {  17, -18 },
    new int[] {  16, -18 },
    new int[] {  15, -18 },
    new int[] {  14, -18 },
    new int[] {  13, -18 },
    new int[] {  12, -18 },
    new int[] {  11, -18 },
    new int[] {  10, -18 },
    new int[] {   9, -18 },
    new int[] {   8, -18 },
    new int[] {   7, -18 },
    new int[] {   6, -18 },
    new int[] {   5, -18 },
    new int[] {   4, -18 },
    new int[] {   3, -18 },
    new int[] {   2, -18 },
    new int[] {   1, -18 },
    new int[] {   0, -18 },
    new int[] {  -1, -18 },
    new int[] {  -2, -18 },
    new int[] {  -3, -18 },
    new int[] {  -4, -18 },
    new int[] {  -5, -18 },
    new int[] {  -6, -18 },
    new int[] {  -7, -18 },
    new int[] {  -8, -18 },
    new int[] {  -9, -18 },
    new int[] { -10, -18 },
    new int[] { -11, -18 },
    new int[] { -12, -18 },
    new int[] { -13, -18 },
    new int[] { -14, -18 },
    new int[] { -15, -18 },
    new int[] { -16, -18 },
    new int[] { -17, -18 },
    new int[] { -18, -18 },
    new int[] { -18, -17 },
    new int[] { -18, -16 },
    new int[] { -18, -15 },
    new int[] { -18, -14 },
    new int[] { -18, -13 },
    new int[] { -18, -12 },
    new int[] { -18, -11 },
    new int[] { -18, -10 },
    new int[] { -18,  -9 },
    new int[] { -18,  -8 },
    new int[] { -18,  -7 },
    new int[] { -18,  -6 },
    new int[] { -18,  -5 },
    new int[] { -18,  -4 },
    new int[] { -18,  -3 },
    new int[] { -18,  -2 },
    new int[] { -18,  -1 },
    new int[] { -18,   0 },
    new int[] { -18,   1 },
    new int[] { -18,   2 },
    new int[] { -18,   3 },
    new int[] { -18,   4 },
    new int[] { -18,   5 },
    new int[] { -18,   6 },
    new int[] { -18,   7 },
    new int[] { -18,   8 },
    new int[] { -18,   9 },
    new int[] { -18,  10 },
    new int[] { -18,  11 },
    new int[] { -18,  12 },
    new int[] { -18,  13 },
    new int[] { -18,  14 },
    new int[] { -18,  15 },
    new int[] { -18,  16 },
    new int[] { -18,  17 },
    new int[] { -18,  18 },
    new int[] { -18,  19 },
    new int[] { -17,  19 },
    new int[] { -16,  19 },
    new int[] { -15,  19 },
    new int[] { -14,  19 },
    new int[] { -13,  19 },
    new int[] { -12,  19 },
    new int[] { -11,  19 },
    new int[] { -10,  19 },
    new int[] {  -9,  19 },
    new int[] {  -8,  19 },
    new int[] {  -7,  19 },
    new int[] {  -6,  19 },
    new int[] {  -5,  19 },
    new int[] {  -4,  19 },
    new int[] {  -3,  19 },
    new int[] {  -2,  19 },
    new int[] {  -1,  19 },
    new int[] {   0,  19 },
    new int[] {   1,  19 },
    new int[] {   2,  19 },
    new int[] {   3,  19 },
    new int[] {   4,  19 },
    new int[] {   5,  19 },
    new int[] {   6,  19 },
    new int[] {   7,  19 },
    new int[] {   8,  19 },
    new int[] {   9,  19 },
    new int[] {  10,  19 },
    new int[] {  11,  19 },
    new int[] {  12,  19 },
    new int[] {  13,  19 },
    new int[] {  14,  19 },
    new int[] {  15,  19 },
    new int[] {  16,  19 },
    new int[] {  17,  19 },
    new int[] {  18,  19 },
    new int[] {  19,  19 },
    new int[] {  19,  18 },
    new int[] {  19,  17 },
    new int[] {  19,  16 },
    new int[] {  19,  15 },
    new int[] {  19,  14 },
    new int[] {  19,  13 },
    new int[] {  19,  12 },
    new int[] {  19,  11 },
    new int[] {  19,  10 },
    new int[] {  19,   9 },
    new int[] {  19,   8 },
    new int[] {  19,   7 },
    new int[] {  19,   6 },
    new int[] {  19,   5 },
    new int[] {  19,   4 },
    new int[] {  19,   3 },
    new int[] {  19,   2 },
    new int[] {  19,   1 },
    new int[] {  19,   0 },
    new int[] {  19,  -1 },
    new int[] {  19,  -2 },
    new int[] {  19,  -3 },
    new int[] {  19,  -4 },
    new int[] {  19,  -5 },
    new int[] {  19,  -6 },
    new int[] {  19,  -7 },
    new int[] {  19,  -8 },
    new int[] {  19,  -9 },
    new int[] {  19, -10 },
    new int[] {  19, -11 },
    new int[] {  19, -12 },
    new int[] {  19, -13 },
    new int[] {  19, -14 },
    new int[] {  19, -15 },
    new int[] {  19, -16 },
    new int[] {  19, -17 },
    new int[] {  19, -18 },
    new int[] {  19, -19 },
    new int[] {  18, -19 },
    new int[] {  17, -19 },
    new int[] {  16, -19 },
    new int[] {  15, -19 },
    new int[] {  14, -19 },
    new int[] {  13, -19 },
    new int[] {  12, -19 },
    new int[] {  11, -19 },
    new int[] {  10, -19 },
    new int[] {   9, -19 },
    new int[] {   8, -19 },
    new int[] {   7, -19 },
    new int[] {   6, -19 },
    new int[] {   5, -19 },
    new int[] {   4, -19 },
    new int[] {   3, -19 },
    new int[] {   2, -19 },
    new int[] {   1, -19 },
    new int[] {   0, -19 },
    new int[] {  -1, -19 },
    new int[] {  -2, -19 },
    new int[] {  -3, -19 },
    new int[] {  -4, -19 },
    new int[] {  -5, -19 },
    new int[] {  -6, -19 },
    new int[] {  -7, -19 },
    new int[] {  -8, -19 },
    new int[] {  -9, -19 },
    new int[] { -10, -19 },
    new int[] { -11, -19 },
    new int[] { -12, -19 },
    new int[] { -13, -19 },
    new int[] { -14, -19 },
    new int[] { -15, -19 },
    new int[] { -16, -19 },
    new int[] { -17, -19 },
    new int[] { -18, -19 },
    new int[] { -19, -19 },
    new int[] { -19, -18 },
    new int[] { -19, -17 },
    new int[] { -19, -16 },
    new int[] { -19, -15 },
    new int[] { -19, -14 },
    new int[] { -19, -13 },
    new int[] { -19, -12 },
    new int[] { -19, -11 },
    new int[] { -19, -10 },
    new int[] { -19,  -9 },
    new int[] { -19,  -8 },
    new int[] { -19,  -7 },
    new int[] { -19,  -6 },
    new int[] { -19,  -5 },
    new int[] { -19,  -4 },
    new int[] { -19,  -3 },
    new int[] { -19,  -2 },
    new int[] { -19,  -1 },
    new int[] { -19,   0 },
    new int[] { -19,   1 },
    new int[] { -19,   2 },
    new int[] { -19,   3 },
    new int[] { -19,   4 },
    new int[] { -19,   5 },
    new int[] { -19,   6 },
    new int[] { -19,   7 },
    new int[] { -19,   8 },
    new int[] { -19,   9 },
    new int[] { -19,  10 },
    new int[] { -19,  11 },
    new int[] { -19,  12 },
    new int[] { -19,  13 },
    new int[] { -19,  14 },
    new int[] { -19,  15 },
    new int[] { -19,  16 },
    new int[] { -19,  17 },
    new int[] { -19,  18 },
    new int[] { -19,  19 },
    new int[] { -19,  20 },
    new int[] { -18,  20 },
    new int[] { -17,  20 },
    new int[] { -16,  20 },
    new int[] { -15,  20 },
    new int[] { -14,  20 },
    new int[] { -13,  20 },
    new int[] { -12,  20 },
    new int[] { -11,  20 },
    new int[] { -10,  20 },
    new int[] {  -9,  20 },
    new int[] {  -8,  20 },
    new int[] {  -7,  20 },
    new int[] {  -6,  20 },
    new int[] {  -5,  20 },
    new int[] {  -4,  20 },
    new int[] {  -3,  20 },
    new int[] {  -2,  20 },
    new int[] {  -1,  20 },
    new int[] {   0,  20 },
    new int[] {   1,  20 },
    new int[] {   2,  20 },
    new int[] {   3,  20 },
    new int[] {   4,  20 },
    new int[] {   5,  20 },
    new int[] {   6,  20 },
    new int[] {   7,  20 },
    new int[] {   8,  20 },
    new int[] {   9,  20 },
    new int[] {  10,  20 },
    new int[] {  11,  20 },
    new int[] {  12,  20 },
    new int[] {  13,  20 },
    new int[] {  14,  20 },
    new int[] {  15,  20 },
    new int[] {  16,  20 },
    new int[] {  17,  20 },
    new int[] {  18,  20 },
    new int[] {  19,  20 },
    new int[] {  20,  20 },
    new int[] {  20,  19 },
    new int[] {  20,  18 },
    new int[] {  20,  17 },
    new int[] {  20,  16 },
    new int[] {  20,  15 },
    new int[] {  20,  14 },
    new int[] {  20,  13 },
    new int[] {  20,  12 },
    new int[] {  20,  11 },
    new int[] {  20,  10 },
    new int[] {  20,   9 },
    new int[] {  20,   8 },
    new int[] {  20,   7 },
    new int[] {  20,   6 },
    new int[] {  20,   5 },
    new int[] {  20,   4 },
    new int[] {  20,   3 },
    new int[] {  20,   2 },
    new int[] {  20,   1 },
    new int[] {  20,   0 },
    new int[] {  20,  -1 },
    new int[] {  20,  -2 },
    new int[] {  20,  -3 },
    new int[] {  20,  -4 },
    new int[] {  20,  -5 },
    new int[] {  20,  -6 },
    new int[] {  20,  -7 },
    new int[] {  20,  -8 },
    new int[] {  20,  -9 },
    new int[] {  20, -10 },
    new int[] {  20, -11 },
    new int[] {  20, -12 },
    new int[] {  20, -13 },
    new int[] {  20, -14 },
    new int[] {  20, -15 },
    new int[] {  20, -16 },
    new int[] {  20, -17 },
    new int[] {  20, -18 },
    new int[] {  20, -19 },
    new int[] {  20, -20 },
    new int[] {  19, -20 },
    new int[] {  18, -20 },
    new int[] {  17, -20 },
    new int[] {  16, -20 },
    new int[] {  15, -20 },
    new int[] {  14, -20 },
    new int[] {  13, -20 },
    new int[] {  12, -20 },
    new int[] {  11, -20 },
    new int[] {  10, -20 },
    new int[] {   9, -20 },
    new int[] {   8, -20 },
    new int[] {   7, -20 },
    new int[] {   6, -20 },
    new int[] {   5, -20 },
    new int[] {   4, -20 },
    new int[] {   3, -20 },
    new int[] {   2, -20 },
    new int[] {   1, -20 },
    new int[] {   0, -20 },
    new int[] {  -1, -20 },
    new int[] {  -2, -20 },
    new int[] {  -3, -20 },
    new int[] {  -4, -20 },
    new int[] {  -5, -20 },
    new int[] {  -6, -20 },
    new int[] {  -7, -20 },
    new int[] {  -8, -20 },
    new int[] {  -9, -20 },
    new int[] { -10, -20 },
    new int[] { -11, -20 },
    new int[] { -12, -20 },
    new int[] { -13, -20 },
    new int[] { -14, -20 },
    new int[] { -15, -20 },
    new int[] { -16, -20 },
    new int[] { -17, -20 },
    new int[] { -18, -20 },
    new int[] { -19, -20 },
    new int[] { -20, -20 },
    new int[] { -20, -19 },
    new int[] { -20, -18 },
    new int[] { -20, -17 },
    new int[] { -20, -16 },
    new int[] { -20, -15 },
    new int[] { -20, -14 },
    new int[] { -20, -13 },
    new int[] { -20, -12 },
    new int[] { -20, -11 },
    new int[] { -20, -10 },
    new int[] { -20,  -9 },
    new int[] { -20,  -8 },
    new int[] { -20,  -7 },
    new int[] { -20,  -6 },
    new int[] { -20,  -5 },
    new int[] { -20,  -4 },
    new int[] { -20,  -3 },
    new int[] { -20,  -2 },
    new int[] { -20,  -1 },
    new int[] { -20,   0 },
    new int[] { -20,   1 },
    new int[] { -20,   2 },
    new int[] { -20,   3 },
    new int[] { -20,   4 },
    new int[] { -20,   5 },
    new int[] { -20,   6 },
    new int[] { -20,   7 },
    new int[] { -20,   8 },
    new int[] { -20,   9 },
    new int[] { -20,  10 },
    new int[] { -20,  11 },
    new int[] { -20,  12 },
    new int[] { -20,  13 },
    new int[] { -20,  14 },
    new int[] { -20,  15 },
    new int[] { -20,  16 },
    new int[] { -20,  17 },
    new int[] { -20,  18 },
    new int[] { -20,  19 },
    new int[] { -20,  20 },
    new int[] { -20,  21 },
    new int[] { -19,  21 },
    new int[] { -18,  21 },
    new int[] { -17,  21 },
    new int[] { -16,  21 },
    new int[] { -15,  21 },
    new int[] { -14,  21 },
    new int[] { -13,  21 },
    new int[] { -12,  21 },
    new int[] { -11,  21 },
    new int[] { -10,  21 },
    new int[] {  -9,  21 },
    new int[] {  -8,  21 },
    new int[] {  -7,  21 },
    new int[] {  -6,  21 },
    new int[] {  -5,  21 },
    new int[] {  -4,  21 },
    new int[] {  -3,  21 },
    new int[] {  -2,  21 },
    new int[] {  -1,  21 },
    new int[] {   0,  21 },
    new int[] {   1,  21 },
    new int[] {   2,  21 },
    new int[] {   3,  21 },
    new int[] {   4,  21 },
    new int[] {   5,  21 },
    new int[] {   6,  21 },
    new int[] {   7,  21 },
    new int[] {   8,  21 },
    new int[] {   9,  21 },
    new int[] {  10,  21 },
    new int[] {  11,  21 },
    new int[] {  12,  21 },
    new int[] {  13,  21 },
    new int[] {  14,  21 },
    new int[] {  15,  21 },
    new int[] {  16,  21 },
    new int[] {  17,  21 },
    new int[] {  18,  21 },
    new int[] {  19,  21 },
    new int[] {  20,  21 },
    new int[] {  21,  21 },
    new int[] {  21,  20 },
    new int[] {  21,  19 },
    new int[] {  21,  18 },
    new int[] {  21,  17 },
    new int[] {  21,  16 },
    new int[] {  21,  15 },
    new int[] {  21,  14 },
    new int[] {  21,  13 },
    new int[] {  21,  12 },
    new int[] {  21,  11 },
    new int[] {  21,  10 },
    new int[] {  21,   9 },
    new int[] {  21,   8 },
    new int[] {  21,   7 },
    new int[] {  21,   6 },
    new int[] {  21,   5 },
    new int[] {  21,   4 },
    new int[] {  21,   3 },
    new int[] {  21,   2 },
    new int[] {  21,   1 },
    new int[] {  21,   0 },
    new int[] {  21,  -1 },
    new int[] {  21,  -2 },
    new int[] {  21,  -3 },
    new int[] {  21,  -4 },
    new int[] {  21,  -5 },
    new int[] {  21,  -6 },
    new int[] {  21,  -7 },
    new int[] {  21,  -8 },
    new int[] {  21,  -9 },
    new int[] {  21, -10 },
    new int[] {  21, -11 },
    new int[] {  21, -12 },
    new int[] {  21, -13 },
    new int[] {  21, -14 },
    new int[] {  21, -15 },
    new int[] {  21, -16 },
    new int[] {  21, -17 },
    new int[] {  21, -18 },
    new int[] {  21, -19 },
    new int[] {  21, -20 },
    new int[] {  21, -21 },
    new int[] {  20, -21 },
    new int[] {  19, -21 },
    new int[] {  18, -21 },
    new int[] {  17, -21 },
    new int[] {  16, -21 },
    new int[] {  15, -21 },
    new int[] {  14, -21 },
    new int[] {  13, -21 },
    new int[] {  12, -21 },
    new int[] {  11, -21 },
    new int[] {  10, -21 },
    new int[] {   9, -21 },
    new int[] {   8, -21 },
    new int[] {   7, -21 },
    new int[] {   6, -21 },
    new int[] {   5, -21 },
    new int[] {   4, -21 },
    new int[] {   3, -21 },
    new int[] {   2, -21 },
    new int[] {   1, -21 },
    new int[] {   0, -21 },
    new int[] {  -1, -21 },
    new int[] {  -2, -21 },
    new int[] {  -3, -21 },
    new int[] {  -4, -21 },
    new int[] {  -5, -21 },
    new int[] {  -6, -21 },
    new int[] {  -7, -21 },
    new int[] {  -8, -21 },
    new int[] {  -9, -21 },
    new int[] { -10, -21 },
    new int[] { -11, -21 },
    new int[] { -12, -21 },
    new int[] { -13, -21 },
    new int[] { -14, -21 },
    new int[] { -15, -21 },
    new int[] { -16, -21 },
    new int[] { -17, -21 },
    new int[] { -18, -21 },
    new int[] { -19, -21 },
    new int[] { -20, -21 },
    new int[] { -21, -21 },
    new int[] { -21, -20 },
    new int[] { -21, -19 },
    new int[] { -21, -18 },
    new int[] { -21, -17 },
    new int[] { -21, -16 },
    new int[] { -21, -15 },
    new int[] { -21, -14 },
    new int[] { -21, -13 },
    new int[] { -21, -12 },
    new int[] { -21, -11 },
    new int[] { -21, -10 },
    new int[] { -21,  -9 },
    new int[] { -21,  -8 },
    new int[] { -21,  -7 },
    new int[] { -21,  -6 },
    new int[] { -21,  -5 },
    new int[] { -21,  -4 },
    new int[] { -21,  -3 },
    new int[] { -21,  -2 },
    new int[] { -21,  -1 },
    new int[] { -21,   0 },
    new int[] { -21,   1 },
    new int[] { -21,   2 },
    new int[] { -21,   3 },
    new int[] { -21,   4 },
    new int[] { -21,   5 },
    new int[] { -21,   6 },
    new int[] { -21,   7 },
    new int[] { -21,   8 },
    new int[] { -21,   9 },
    new int[] { -21,  10 },
    new int[] { -21,  11 },
    new int[] { -21,  12 },
    new int[] { -21,  13 },
    new int[] { -21,  14 },
    new int[] { -21,  15 },
    new int[] { -21,  16 },
    new int[] { -21,  17 },
    new int[] { -21,  18 },
    new int[] { -21,  19 },
    new int[] { -21,  20 },
    new int[] { -21,  21 },
    new int[] { -21,  22 },
    new int[] { -20,  22 },
    new int[] { -19,  22 },
    new int[] { -18,  22 },
    new int[] { -17,  22 },
    new int[] { -16,  22 },
    new int[] { -15,  22 },
    new int[] { -14,  22 },
    new int[] { -13,  22 },
    new int[] { -12,  22 },
    new int[] { -11,  22 },
    new int[] { -10,  22 },
    new int[] {  -9,  22 },
    new int[] {  -8,  22 },
    new int[] {  -7,  22 },
    new int[] {  -6,  22 },
    new int[] {  -5,  22 },
    new int[] {  -4,  22 },
    new int[] {  -3,  22 },
    new int[] {  -2,  22 },
    new int[] {  -1,  22 },
    new int[] {   0,  22 },
    new int[] {   1,  22 },
    new int[] {   2,  22 },
    new int[] {   3,  22 },
    new int[] {   4,  22 },
    new int[] {   5,  22 },
    new int[] {   6,  22 },
    new int[] {   7,  22 },
    new int[] {   8,  22 },
    new int[] {   9,  22 },
    new int[] {  10,  22 },
    new int[] {  11,  22 },
    new int[] {  12,  22 },
    new int[] {  13,  22 },
    new int[] {  14,  22 },
    new int[] {  15,  22 },
    new int[] {  16,  22 },
    new int[] {  17,  22 },
    new int[] {  18,  22 },
    new int[] {  19,  22 },
    new int[] {  20,  22 },
    new int[] {  21,  22 },
    new int[] {  22,  22 },
    new int[] {  22,  21 },
    new int[] {  22,  20 },
    new int[] {  22,  19 },
    new int[] {  22,  18 },
    new int[] {  22,  17 },
    new int[] {  22,  16 },
    new int[] {  22,  15 },
    new int[] {  22,  14 },
    new int[] {  22,  13 },
    new int[] {  22,  12 },
    new int[] {  22,  11 },
    new int[] {  22,  10 },
    new int[] {  22,   9 },
    new int[] {  22,   8 },
    new int[] {  22,   7 },
    new int[] {  22,   6 },
    new int[] {  22,   5 },
    new int[] {  22,   4 },
    new int[] {  22,   3 },
    new int[] {  22,   2 },
    new int[] {  22,   1 },
    new int[] {  22,   0 },
    new int[] {  22,  -1 },
    new int[] {  22,  -2 },
    new int[] {  22,  -3 },
    new int[] {  22,  -4 },
    new int[] {  22,  -5 },
    new int[] {  22,  -6 },
    new int[] {  22,  -7 },
    new int[] {  22,  -8 },
    new int[] {  22,  -9 },
    new int[] {  22, -10 },
    new int[] {  22, -11 },
    new int[] {  22, -12 },
    new int[] {  22, -13 },
    new int[] {  22, -14 },
    new int[] {  22, -15 },
    new int[] {  22, -16 },
    new int[] {  22, -17 },
    new int[] {  22, -18 },
    new int[] {  22, -19 },
    new int[] {  22, -20 },
    new int[] {  22, -21 },
    new int[] {  22, -22 },
    new int[] {  21, -22 },
    new int[] {  20, -22 },
    new int[] {  19, -22 },
    new int[] {  18, -22 },
    new int[] {  17, -22 },
    new int[] {  16, -22 },
    new int[] {  15, -22 },
    new int[] {  14, -22 },
    new int[] {  13, -22 },
    new int[] {  12, -22 },
    new int[] {  11, -22 },
    new int[] {  10, -22 },
    new int[] {   9, -22 },
    new int[] {   8, -22 },
    new int[] {   7, -22 },
    new int[] {   6, -22 },
    new int[] {   5, -22 },
    new int[] {   4, -22 },
    new int[] {   3, -22 },
    new int[] {   2, -22 },
    new int[] {   1, -22 },
    new int[] {   0, -22 },
    new int[] {  -1, -22 },
    new int[] {  -2, -22 },
    new int[] {  -3, -22 },
    new int[] {  -4, -22 },
    new int[] {  -5, -22 },
    new int[] {  -6, -22 },
    new int[] {  -7, -22 },
    new int[] {  -8, -22 },
    new int[] {  -9, -22 },
    new int[] { -10, -22 },
    new int[] { -11, -22 },
    new int[] { -12, -22 },
    new int[] { -13, -22 },
    new int[] { -14, -22 },
    new int[] { -15, -22 },
    new int[] { -16, -22 },
    new int[] { -17, -22 },
    new int[] { -18, -22 },
    new int[] { -19, -22 },
    new int[] { -20, -22 },
    new int[] { -21, -22 },
    new int[] { -22, -22 },
    new int[] { -22, -21 },
    new int[] { -22, -20 },
    new int[] { -22, -19 },
    new int[] { -22, -18 },
    new int[] { -22, -17 },
    new int[] { -22, -16 },
    new int[] { -22, -15 },
    new int[] { -22, -14 },
    new int[] { -22, -13 },
    new int[] { -22, -12 },
    new int[] { -22, -11 },
    new int[] { -22, -10 },
    new int[] { -22,  -9 },
    new int[] { -22,  -8 },
    new int[] { -22,  -7 },
    new int[] { -22,  -6 },
    new int[] { -22,  -5 },
    new int[] { -22,  -4 },
    new int[] { -22,  -3 },
    new int[] { -22,  -2 },
    new int[] { -22,  -1 },
    new int[] { -22,   0 },
    new int[] { -22,   1 },
    new int[] { -22,   2 },
    new int[] { -22,   3 },
    new int[] { -22,   4 },
    new int[] { -22,   5 },
    new int[] { -22,   6 },
    new int[] { -22,   7 },
    new int[] { -22,   8 },
    new int[] { -22,   9 },
    new int[] { -22,  10 },
    new int[] { -22,  11 },
    new int[] { -22,  12 },
    new int[] { -22,  13 },
    new int[] { -22,  14 },
    new int[] { -22,  15 },
    new int[] { -22,  16 },
    new int[] { -22,  17 },
    new int[] { -22,  18 },
    new int[] { -22,  19 },
    new int[] { -22,  20 },
    new int[] { -22,  21 },
    new int[] { -22,  22 },
    new int[] { -22,  23 },
    new int[] { -21,  23 },
    new int[] { -20,  23 },
    new int[] { -19,  23 },
    new int[] { -18,  23 },
    new int[] { -17,  23 },
    new int[] { -16,  23 },
    new int[] { -15,  23 },
    new int[] { -14,  23 },
    new int[] { -13,  23 },
    new int[] { -12,  23 },
    new int[] { -11,  23 },
    new int[] { -10,  23 },
    new int[] {  -9,  23 },
    new int[] {  -8,  23 },
    new int[] {  -7,  23 },
    new int[] {  -6,  23 },
    new int[] {  -5,  23 },
    new int[] {  -4,  23 },
    new int[] {  -3,  23 },
    new int[] {  -2,  23 },
    new int[] {  -1,  23 },
    new int[] {   0,  23 },
    new int[] {   1,  23 },
    new int[] {   2,  23 },
    new int[] {   3,  23 },
    new int[] {   4,  23 },
    new int[] {   5,  23 },
    new int[] {   6,  23 },
    new int[] {   7,  23 },
    new int[] {   8,  23 },
    new int[] {   9,  23 },
    new int[] {  10,  23 },
    new int[] {  11,  23 },
    new int[] {  12,  23 },
    new int[] {  13,  23 },
    new int[] {  14,  23 },
    new int[] {  15,  23 },
    new int[] {  16,  23 },
    new int[] {  17,  23 },
    new int[] {  18,  23 },
    new int[] {  19,  23 },
    new int[] {  20,  23 },
    new int[] {  21,  23 },
    new int[] {  22,  23 },
    new int[] {  23,  23 },
    new int[] {  23,  22 },
    new int[] {  23,  21 },
    new int[] {  23,  20 },
    new int[] {  23,  19 },
    new int[] {  23,  18 },
    new int[] {  23,  17 },
    new int[] {  23,  16 },
    new int[] {  23,  15 },
    new int[] {  23,  14 },
    new int[] {  23,  13 },
    new int[] {  23,  12 },
    new int[] {  23,  11 },
    new int[] {  23,  10 },
    new int[] {  23,   9 },
    new int[] {  23,   8 },
    new int[] {  23,   7 },
    new int[] {  23,   6 },
    new int[] {  23,   5 },
    new int[] {  23,   4 },
    new int[] {  23,   3 },
    new int[] {  23,   2 },
    new int[] {  23,   1 },
    new int[] {  23,   0 },
    new int[] {  23,  -1 },
    new int[] {  23,  -2 },
    new int[] {  23,  -3 },
    new int[] {  23,  -4 },
    new int[] {  23,  -5 },
    new int[] {  23,  -6 },
    new int[] {  23,  -7 },
    new int[] {  23,  -8 },
    new int[] {  23,  -9 },
    new int[] {  23, -10 },
    new int[] {  23, -11 },
    new int[] {  23, -12 },
    new int[] {  23, -13 },
    new int[] {  23, -14 },
    new int[] {  23, -15 },
    new int[] {  23, -16 },
    new int[] {  23, -17 },
    new int[] {  23, -18 },
    new int[] {  23, -19 },
    new int[] {  23, -20 },
    new int[] {  23, -21 },
    new int[] {  23, -22 },
    new int[] {  23, -23 },
    new int[] {  22, -23 },
    new int[] {  21, -23 },
    new int[] {  20, -23 },
    new int[] {  19, -23 },
    new int[] {  18, -23 },
    new int[] {  17, -23 },
    new int[] {  16, -23 },
    new int[] {  15, -23 },
    new int[] {  14, -23 },
    new int[] {  13, -23 },
    new int[] {  12, -23 },
    new int[] {  11, -23 },
    new int[] {  10, -23 },
    new int[] {   9, -23 },
    new int[] {   8, -23 },
    new int[] {   7, -23 },
    new int[] {   6, -23 },
    new int[] {   5, -23 },
    new int[] {   4, -23 },
    new int[] {   3, -23 },
    new int[] {   2, -23 },
    new int[] {   1, -23 },
    new int[] {   0, -23 },
    new int[] {  -1, -23 },
    new int[] {  -2, -23 },
    new int[] {  -3, -23 },
    new int[] {  -4, -23 },
    new int[] {  -5, -23 },
    new int[] {  -6, -23 },
    new int[] {  -7, -23 },
    new int[] {  -8, -23 },
    new int[] {  -9, -23 },
    new int[] { -10, -23 },
    new int[] { -11, -23 },
    new int[] { -12, -23 },
    new int[] { -13, -23 },
    new int[] { -14, -23 },
    new int[] { -15, -23 },
    new int[] { -16, -23 },
    new int[] { -17, -23 },
    new int[] { -18, -23 },
    new int[] { -19, -23 },
    new int[] { -20, -23 },
    new int[] { -21, -23 },
    new int[] { -22, -23 },
    new int[] { -23, -23 },
    new int[] { -23, -22 },
    new int[] { -23, -21 },
    new int[] { -23, -20 },
    new int[] { -23, -19 },
    new int[] { -23, -18 },
    new int[] { -23, -17 },
    new int[] { -23, -16 },
    new int[] { -23, -15 },
    new int[] { -23, -14 },
    new int[] { -23, -13 },
    new int[] { -23, -12 },
    new int[] { -23, -11 },
    new int[] { -23, -10 },
    new int[] { -23,  -9 },
    new int[] { -23,  -8 },
    new int[] { -23,  -7 },
    new int[] { -23,  -6 },
    new int[] { -23,  -5 },
    new int[] { -23,  -4 },
    new int[] { -23,  -3 },
    new int[] { -23,  -2 },
    new int[] { -23,  -1 },
    new int[] { -23,   0 },
    new int[] { -23,   1 },
    new int[] { -23,   2 },
    new int[] { -23,   3 },
    new int[] { -23,   4 },
    new int[] { -23,   5 },
    new int[] { -23,   6 },
    new int[] { -23,   7 },
    new int[] { -23,   8 },
    new int[] { -23,   9 },
    new int[] { -23,  10 },
    new int[] { -23,  11 },
    new int[] { -23,  12 },
    new int[] { -23,  13 },
    new int[] { -23,  14 },
    new int[] { -23,  15 },
    new int[] { -23,  16 },
    new int[] { -23,  17 },
    new int[] { -23,  18 },
    new int[] { -23,  19 },
    new int[] { -23,  20 },
    new int[] { -23,  21 },
    new int[] { -23,  22 },
    new int[] { -23,  23 },
    new int[] { -23,  24 },
    new int[] { -22,  24 },
    new int[] { -21,  24 },
    new int[] { -20,  24 },
    new int[] { -19,  24 },
    new int[] { -18,  24 },
    new int[] { -17,  24 },
    new int[] { -16,  24 },
    new int[] { -15,  24 },
    new int[] { -14,  24 },
    new int[] { -13,  24 },
    new int[] { -12,  24 },
    new int[] { -11,  24 },
    new int[] { -10,  24 },
    new int[] {  -9,  24 },
    new int[] {  -8,  24 },
    new int[] {  -7,  24 },
    new int[] {  -6,  24 },
    new int[] {  -5,  24 },
    new int[] {  -4,  24 },
    new int[] {  -3,  24 },
    new int[] {  -2,  24 },
    new int[] {  -1,  24 },
    new int[] {   0,  24 },
    new int[] {   1,  24 },
    new int[] {   2,  24 },
    new int[] {   3,  24 },
    new int[] {   4,  24 },
    new int[] {   5,  24 },
    new int[] {   6,  24 },
    new int[] {   7,  24 },
    new int[] {   8,  24 },
    new int[] {   9,  24 },
    new int[] {  10,  24 },
    new int[] {  11,  24 },
    new int[] {  12,  24 },
    new int[] {  13,  24 },
    new int[] {  14,  24 },
    new int[] {  15,  24 },
    new int[] {  16,  24 },
    new int[] {  17,  24 },
    new int[] {  18,  24 },
    new int[] {  19,  24 },
    new int[] {  20,  24 },
    new int[] {  21,  24 },
    new int[] {  22,  24 },
    new int[] {  23,  24 },
    new int[] {  24,  24 },
    new int[] {  24,  23 },
    new int[] {  24,  22 },
    new int[] {  24,  21 },
    new int[] {  24,  20 },
    new int[] {  24,  19 },
    new int[] {  24,  18 },
    new int[] {  24,  17 },
    new int[] {  24,  16 },
    new int[] {  24,  15 },
    new int[] {  24,  14 },
    new int[] {  24,  13 },
    new int[] {  24,  12 },
    new int[] {  24,  11 },
    new int[] {  24,  10 },
    new int[] {  24,   9 },
    new int[] {  24,   8 },
    new int[] {  24,   7 },
    new int[] {  24,   6 },
    new int[] {  24,   5 },
    new int[] {  24,   4 },
    new int[] {  24,   3 },
    new int[] {  24,   2 },
    new int[] {  24,   1 },
    new int[] {  24,   0 },
    new int[] {  24,  -1 },
    new int[] {  24,  -2 },
    new int[] {  24,  -3 },
    new int[] {  24,  -4 },
    new int[] {  24,  -5 },
    new int[] {  24,  -6 },
    new int[] {  24,  -7 },
    new int[] {  24,  -8 },
    new int[] {  24,  -9 },
    new int[] {  24, -10 },
    new int[] {  24, -11 },
    new int[] {  24, -12 },
    new int[] {  24, -13 },
    new int[] {  24, -14 },
    new int[] {  24, -15 },
    new int[] {  24, -16 },
    new int[] {  24, -17 },
    new int[] {  24, -18 },
    new int[] {  24, -19 },
    new int[] {  24, -20 },
    new int[] {  24, -21 },
    new int[] {  24, -22 },
    new int[] {  24, -23 },
    new int[] {  24, -24 },
    new int[] {  23, -24 },
    new int[] {  22, -24 },
    new int[] {  21, -24 },
    new int[] {  20, -24 },
    new int[] {  19, -24 },
    new int[] {  18, -24 },
    new int[] {  17, -24 },
    new int[] {  16, -24 },
    new int[] {  15, -24 },
    new int[] {  14, -24 },
    new int[] {  13, -24 },
    new int[] {  12, -24 },
    new int[] {  11, -24 },
    new int[] {  10, -24 },
    new int[] {   9, -24 },
    new int[] {   8, -24 },
    new int[] {   7, -24 },
    new int[] {   6, -24 },
    new int[] {   5, -24 },
    new int[] {   4, -24 },
    new int[] {   3, -24 },
    new int[] {   2, -24 },
    new int[] {   1, -24 },
    new int[] {   0, -24 },
    new int[] {  -1, -24 },
    new int[] {  -2, -24 },
    new int[] {  -3, -24 },
    new int[] {  -4, -24 },
    new int[] {  -5, -24 },
    new int[] {  -6, -24 },
    new int[] {  -7, -24 },
    new int[] {  -8, -24 },
    new int[] {  -9, -24 },
    new int[] { -10, -24 },
    new int[] { -11, -24 },
    new int[] { -12, -24 },
    new int[] { -13, -24 },
    new int[] { -14, -24 },
    new int[] { -15, -24 },
    new int[] { -16, -24 },
    new int[] { -17, -24 },
    new int[] { -18, -24 },
    new int[] { -19, -24 },
    new int[] { -20, -24 },
    new int[] { -21, -24 },
    new int[] { -22, -24 },
    new int[] { -23, -24 },
    new int[] { -24, -24 },
};
}
//--------------------------------------------------------------------------------------------------
} // namespace Searches
//--------------------------------------------------------------------------------------------------
